@@ -1,8 +1,8 @@
 /* tifffastcrop
 
- v. 1.4.1
+ v. 1.4.2
 
- Copyright (c) 2013-2021 Christophe Deroulers
+ Copyright (c) 2013-2025 Christophe Deroulers
 
  Portions are based on libtiff's tiffcp code. tiffcp is:
  Copyright (c) 1988-1997 Sam Leffler
@@ -18,6 +18,7 @@
 #include <string.h>
 #include <strings.h> /* strncasecmp */
 #include <ctype.h> /* isdigit */
+#include <inttypes.h>
 #include <assert.h>
 #include <errno.h>
 #include <tiff.h>
@@ -543,7 +544,7 @@ static int testAndFixOutTIFFPhotoAndCompressionParameters(TIFF* in,
 
 
 static int cpTiles2Strip(TIFF* in, uint32_t xmin, uint32_t ymin,
-	uint32_t width, uint32_t length, unsigned char * outbuf,
+	uint32_t width, uint32_t length, uint8_t * outbuf,
 	tsize_t outscanlinesizeinbytes, uint16_t bitspersample,
 	uint16_t samplesperpixel)
 {
@@ -551,14 +552,14 @@ static int cpTiles2Strip(TIFF* in, uint32_t xmin, uint32_t ymin,
 	uint32_t intilewidth = (uint32_t) -1, intilelength = (uint32_t) -1;
 	tsize_t intilewidthinbytes = TIFFTileRowSize(in);
 	uint32_t y;
-	unsigned char * inbuf, * bufp= outbuf;
+	uint8_t * inbuf, * bufp= outbuf;
 	int error = 0;
 
 	TIFFGetField(in, TIFFTAG_TILEWIDTH, &intilewidth);
 	TIFFGetField(in, TIFFTAG_TILELENGTH, &intilelength);
 
 	inbufsize= TIFFTileSize(in);
-	inbuf = (unsigned char *)_TIFFmalloc(inbufsize); /* not malloc
+	inbuf = (uint8_t *) _TIFFmalloc(inbufsize); /* not malloc
 	    because TIFFTileSize returns a tmsize_t */
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
@@ -572,7 +573,7 @@ static int cpTiles2Strip(TIFF* in, uint32_t xmin, uint32_t ymin,
 		uint32_t ymintocopy = ymin > yminoftile ? ymin : yminoftile;
 		uint32_t ymaxplusone = yminoftile + intilelength;
 		uint32_t lengthtocopy;
-		unsigned char * inbufrow = inbuf +
+		uint8_t * inbufrow = inbuf +
 		    intilewidthinbytes * (ymintocopy-yminoftile);
 
 		if (ymaxplusone > ymin + length)
@@ -600,7 +601,7 @@ static int cpTiles2Strip(TIFF* in, uint32_t xmin, uint32_t ymin,
 			    yminoftile, 0, 0) < 0) {
 				TIFFError(TIFFFileName(in),
 				    "Error, can't read tile at "
-				    UINT32_FORMAT ", " UINT32_FORMAT,
+				    "%" PRIu32 ", %" PRIu32,
 				    xminoftile, yminoftile);
 				error = EXIT_IO_ERROR;
 				goto done;
@@ -624,7 +625,7 @@ static int cpTiles2Strip(TIFF* in, uint32_t xmin, uint32_t ymin,
 
 static int cpStrips2Strip(TIFF* in,
 	uint32_t xmin, uint32_t ymin, uint32_t width, uint32_t length,
-	unsigned char * outbuf, tsize_t outscanlinesizeinbytes,
+	uint8_t * outbuf, tsize_t outscanlinesizeinbytes,
 	uint16_t bitspersample, uint16_t samplesperpixel,
 	uint32_t * y_of_last_read_scanline, uint32_t inimagelength)
 {
@@ -632,13 +633,13 @@ static int cpStrips2Strip(TIFF* in,
 	uint16_t input_compression;
 	tsize_t inwidthinbytes = TIFFScanlineSize(in);
 	uint32_t y;
-	unsigned char * inbuf, * bufp= outbuf;
+	uint8_t * inbuf, * bufp= outbuf;
 	int error = 0;
 
 	TIFFGetFieldDefaulted(in, TIFFTAG_COMPRESSION, &input_compression);
 	inbufsize= TIFFScanlineSize(in);  /* not malloc because
 	    TIFFScanlineSize returns a tmsize_t */
-	inbuf = (unsigned char *)_TIFFmalloc(inbufsize);
+	inbuf = (uint8_t *) _TIFFmalloc(inbufsize);
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for image buffer");
@@ -657,7 +658,7 @@ static int cpStrips2Strip(TIFF* in,
 				if (TIFFReadScanline(in, inbuf, y, 0) < 0) {
 					TIFFError(TIFFFileName(in),
 					    "Error, can't read scanline at "
-					    UINT32_FORMAT " for exhausting",
+					    "%" PRIu32 " for exhausting",
 					    y);
 					error = EXIT_IO_ERROR;
 					goto done;
@@ -671,7 +672,7 @@ static int cpStrips2Strip(TIFF* in,
 			if (TIFFReadScanline(in, inbuf, y, 0) < 0) {
 				TIFFError(TIFFFileName(in),
 				    "Error, can't read scanline at "
-				    UINT32_FORMAT " for exhausting", y);
+				    "%" PRIu32 " for exhausting", y);
 				error = EXIT_IO_ERROR;
 				goto done;
 			} else
@@ -684,7 +685,7 @@ static int cpStrips2Strip(TIFF* in,
 		if (TIFFReadScanline(in, inbuf, y, 0) < 0) {
 			TIFFError(TIFFFileName(in),
 			    "Error, can't read scanline at "
-			    UINT32_FORMAT " for copying", y);
+			    "%" PRIu32 " for copying", y);
 			error = EXIT_IO_ERROR;
 			goto done;
 		} else
@@ -735,7 +736,7 @@ static int makeExtractFromTIFFDirectory(const char * infilename, TIFF * in,
 	uint16_t planarconfig, spp, bitspersample;
 	size_t outmemorysize;
 	char * ouroutfilename = NULL;
-	unsigned char * outbuf = NULL;
+	uint8_t * outbuf = NULL;
 	void * out; /* TIFF* or FILE* */
 	uint32_t y_of_last_read_scanline = 0;
 	int return_code = 0; /* Success */
@@ -811,8 +812,8 @@ static int makeExtractFromTIFFDirectory(const char * infilename, TIFF * in,
 	if (requestedlength == (uint32_t) -1)
 		requestedlength = inimagelength - requestedymin;
 	if (verbose)
-		fprintf(stderr, "Requested rectangle: " UINT32_FORMAT
-			"x" UINT32_FORMAT ".\n",
+		fprintf(stderr, "Requested rectangle: %" PRIu32
+			"x%" PRIu32 ".\n",
 			requestedwidth, requestedlength);
 
 	if (requestedxmin > inimagewidth || requestedymin > inimagelength) {
@@ -828,8 +829,8 @@ static int makeExtractFromTIFFDirectory(const char * infilename, TIFF * in,
 		if (verbose)
 			fprintf(stderr, "Requested rectangle extends "
 				"outside the image. Adjusting "
-				"dimensions to " UINT32_FORMAT "x" 
-				UINT32_FORMAT ".\n",
+				"dimensions to %" PRIu32 "x" 
+				"%" PRIu32 ".\n",
 				requestedwidth, requestedlength);
 	}
 
@@ -852,7 +853,7 @@ static int makeExtractFromTIFFDirectory(const char * infilename, TIFF * in,
 	outwidth = requestedwidth; outlength = requestedlength;
 	outmemorysize= computeMemorySize(spp, bitspersample, outwidth,
 		outlength);
-	outbuf= outmemorysize == 0 ? NULL : malloc(outmemorysize);
+	outbuf = outmemorysize == 0 ? NULL : malloc(outmemorysize);
 	if (outbuf == NULL) {
 		fprintf(stderr, "Unable to allocate enough memory to"
 			" prepare extract (%zu bytes needed).\n",
@@ -867,7 +868,7 @@ static int makeExtractFromTIFFDirectory(const char * infilename, TIFF * in,
 		uint32_t ndigitsx = searchNumberOfDigits(inimagewidth),
 		    ndigitsy = searchNumberOfDigits(inimagelength);
 		if (diroff)
-			my_asprintf(&ouroutfilename, "%s-d0x" UINT64_HEX_FORMAT "-%0*u-%0*u-%0*ux%0*u.%s",
+			my_asprintf(&ouroutfilename, "%s-d0x%" PRIx64 "-%0*u-%0*u-%0*ux%0*u.%s",
 			    prefix, diroff, ndigitsx, requestedxmin,
 			    ndigitsy, requestedymin, ndigitsx,
 			    requestedwidth, ndigitsy, requestedlength,
@@ -901,8 +902,8 @@ static int makeExtractFromTIFFDirectory(const char * infilename, TIFF * in,
 			else
 				fprintf(stderr, "Output file \"%s\" open."
 					" Will write extract of size "
-					UINT32_FORMAT " x "
-					UINT32_FORMAT ".\n",
+					"%" PRIu32 " x "
+					"%" PRIu32 ".\n",
 					outfilename,
 					requestedwidth, requestedlength);
 	}
@@ -1206,7 +1207,7 @@ static int makeExtractFromTIFFFile(const char * infilename,
 
 static void usage()
 {
-	fprintf(stderr, "tifffastcrop v" PACKAGE_VERSION " license GNU GPL v3 (c) 2013-2021 Christophe Deroulers\n\n");
+	fprintf(stderr, "tifffastcrop v" PACKAGE_VERSION " license GNU GPL v3 (c) 2013-2025 Christophe Deroulers\n\n");
 	fprintf(stderr, "Quote \"Deroulers et al., Diagnostic Pathology 2013, 8:92\" in your production\n       http://doi.org/10.1186/1746-1596-8-92\n\n");
 	fprintf(stderr, "Usage: tifffastcrop [options] input.tif [output_name]\n\n");
 	fprintf(stderr, " Extracts (crops), without loading the full image input.tif into memory, a\nrectangular region from it, and saves it. Output file name is output_name if\ngiven, otherwise a name derived from input.tif. Output file format is guessed\nfrom output_name's extension if possible. Options:\n");
@@ -1358,8 +1359,8 @@ static int processExtractGeometryOptions(char* cp)
 {
 	while (*cp == ' ')
 		cp++;
-	if (sscanf(cp, UINT32_FORMAT "," UINT32_FORMAT ","
-	    UINT32_FORMAT "," UINT32_FORMAT,
+	if (sscanf(cp, "%" PRIu32 ",%" PRIu32 ","
+	    "%" PRIu32 ",%" PRIu32,
 	    &requestedxmin, &requestedymin, &requestedwidth, 
 	    &requestedlength) != 4)
 		return 0;
