@@ -1,8 +1,8 @@
 /* tiffmakemosaic
 
- v. 1.4.1
+ v. 1.4.2
 
- Copyright (c) 2012-2021 Christophe Deroulers
+ Copyright (c) 2012-2025 Christophe Deroulers
 
  Portions are based on libtiff's tiffcp code. tiffcp is
  Copyright (c) 1988-1997 Sam Leffler
@@ -18,6 +18,7 @@
 #include <stdlib.h> /* exit, strtoul */
 #include <string.h>
 #include <strings.h> /* strncasecmp */
+#include <inttypes.h>
 #include <assert.h>
 #include <errno.h>
 #include <tiff.h>
@@ -374,9 +375,8 @@ static int
 cpTiles2Strip(TIFF* in, void * ambiguous_out,
 	int output_to_jpeg_rather_than_tiff,
 	uint32_t xmin, uint32_t ymin, uint32_t width, uint32_t length,
-	uint32_t rightpadding, uint32_t bottompadding, uint8_t* paddingbytes,
-	uint32_t inimagewidth, uint32_t inimagelength,
-	unsigned char * outbuf)
+	uint32_t rightpadding, uint32_t bottompadding, uint8_t * paddingbytes,
+	uint32_t inimagewidth, uint32_t inimagelength, uint8_t * outbuf)
 {
 	struct jpeg_compress_struct * p_cinfo;
 	TIFF* TIFFout;
@@ -386,7 +386,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 	tsize_t intilewidthinbytes = TIFFTileRowSize(in);
 	tsize_t outscanlinesizeinbytes;
 	uint32_t y;
-	unsigned char * inbuf, * bufp= outbuf;
+	uint8_t * inbuf, * bufp= outbuf;
 	int error = 0;
 
 	if (output_to_jpeg_rather_than_tiff) {
@@ -406,7 +406,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 		return 0;
 
 	inbufsize= TIFFTileSize(in);
-	inbuf = (unsigned char *)_TIFFmalloc(inbufsize);
+	inbuf = (uint8_t *) _TIFFmalloc(inbufsize);
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for image buffer");
@@ -419,7 +419,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 		uint32_t ymintocopyintile = ymin > yminoftile ? ymin : yminoftile;
 		uint32_t ymaxplusone = yminoftile + intilelength;
 		uint32_t lengthtocopy, bottompaddingthistile = 0;
-		unsigned char * inbufrow = inbuf +
+		uint8_t * inbufrow = inbuf +
 		    intilewidthinbytes * (ymintocopyintile - yminoftile);
 
 		if (ymaxplusone > ymin + length)
@@ -465,7 +465,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 			    yminoftile, 0, 0) < 0) {
 				TIFFError(TIFFFileName(in),
 				    "Error, can't read tile at "
-				    UINT32_FORMAT ", " UINT32_FORMAT,
+				    "%" PRIu32 ", %" PRIu32,
 				    xminoftile, yminoftile);
 				error = EXIT_IO_ERROR;
 				goto done;
@@ -525,7 +525,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 	int output_to_jpeg_rather_than_tiff,
 	uint32_t xmin, uint32_t ymin, uint32_t width, uint32_t length,
 	uint32_t rightpadding, uint32_t bottompadding,
-	unsigned char * outbuf, uint32_t * y_of_last_read_scanline,
+	uint8_t * outbuf, uint32_t * y_of_last_read_scanline,
 	uint32_t inimagelength)
 {
 	struct jpeg_compress_struct * p_cinfo;
@@ -535,7 +535,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 	tsize_t inwidthinbytes = TIFFScanlineSize(in);
 	tsize_t outscanlinesizeinbytes;
 	uint32_t y;
-	unsigned char * inbuf, * bufp= outbuf;
+	uint8_t * inbuf, * bufp= outbuf;
 	int error = 0;
 
 	if (output_to_jpeg_rather_than_tiff) {
@@ -552,7 +552,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 		return 0;
 
 	inbufsize= TIFFScanlineSize(in);
-	inbuf = (unsigned char *)_TIFFmalloc(inbufsize);
+	inbuf = (uint8_t *) _TIFFmalloc(inbufsize);
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for image buffer");
@@ -573,7 +573,7 @@ fprintf(stderr, "cpStrips2Strip warning: padding is yet to be implemented\n");
 				if (TIFFReadScanline(in, inbuf, y, 0) < 0) {
 					TIFFError(TIFFFileName(in),
 					    "Error, can't read scanline at "
-					    UINT32_FORMAT " for exhausting",
+					    "%" PRIu32 " for exhausting",
 					    y);
 					error = EXIT_IO_ERROR;
 					goto done;
@@ -587,7 +587,7 @@ fprintf(stderr, "cpStrips2Strip warning: padding is yet to be implemented\n");
 			if (TIFFReadScanline(in, inbuf, y, 0) < 0) {
 				TIFFError(TIFFFileName(in),
 				    "Error, can't read scanline at "
-				    UINT32_FORMAT " for exhausting", y);
+				    "%" PRIu32 " for exhausting", y);
 				error = EXIT_IO_ERROR;
 				goto done;
 			} else
@@ -595,14 +595,14 @@ fprintf(stderr, "cpStrips2Strip warning: padding is yet to be implemented\n");
 	}
 
 	for (y = ymin ; y < ymin + length ; y++) {
-		unsigned char * inbufrow = inbuf;
+		uint8_t * inbufrow = inbuf;
 		uint32_t xmintocopyinscanline = xmin;
 		tsize_t widthtocopyinbytes = outscanlinesizeinbytes;
 
 		if (TIFFReadScanline(in, inbuf, y, 0) < 0) {
 			TIFFError(TIFFFileName(in),
 			    "Error, can't read scanline at "
-			    UINT32_FORMAT " for copying", y);
+			    "%" PRIu32 " for copying", y);
 			error = EXIT_IO_ERROR;
 			goto done;
 		} else
@@ -675,8 +675,10 @@ computeMaxPieceMemorySize(uint32_t inimagewidth, uint32_t inimagelength,
 	if (*voverlap > outpiecelength)
 		*voverlap= outpiecelength;
 
-	*hnpieces = (inimagewidth+outpiecewidth-1) / outpiecewidth;
-	*vnpieces = (inimagelength+outpiecelength-1) / outpiecelength;
+	*hnpieces = outpiecewidth == 0 ? 1 :
+		(inimagewidth+outpiecewidth-1) / outpiecewidth;
+	*vnpieces = outpiecelength == 0 ? 1 :
+		(inimagelength+outpiecelength-1) / outpiecelength;
 
 	maxpiecewidthwithoverlap= outpiecewidth + *hoverlap * (
 		*hnpieces >= 3 ? 2 : *hnpieces-1);
@@ -689,20 +691,20 @@ computeMaxPieceMemorySize(uint32_t inimagewidth, uint32_t inimagelength,
 	*ourmaxoutmemorysize *= spp;
 
 	if (verbose) {
-		fprintf(stderr, "Trying with pieces of " UINT32_FORMAT
-			" x " UINT32_FORMAT " and horiz. overlap "
-			UINT32_FORMAT ", vert. overlap " UINT32_FORMAT
+		fprintf(stderr, "Trying with pieces of %" PRIu32
+			" x %" PRIu32 " and horiz. overlap "
+			"%" PRIu32 ", vert. overlap %" PRIu32
 			" (would need %.3f MiB per piece)",
 			outpiecewidth, outpiecelength, *hoverlap,
 			*voverlap, *maxoutmemorysize / 1048576.0);
 		if (requestedpiecewidthdivisor)
 			fprintf(stderr, ", width %g times the divisor "
-				UINT32_FORMAT,
+				"%" PRIu32,
 				1. * outpiecewidth / requestedpiecewidthdivisor,
 				requestedpiecewidthdivisor);
 		if (requestedpiecelengthdivisor)
 			fprintf(stderr, ", length %g times the divisor "
-				UINT32_FORMAT,
+				"%" PRIu32,
 				1. * outpiecelength /
 				    requestedpiecelengthdivisor,
 				requestedpiecelengthdivisor);
@@ -722,7 +724,7 @@ makeMosaicFromTIFFFile(char * infilename)
 	uint16_t planarconfig, spp, bitspersample, sampleformat;
 	uint64_t outmemorysize, ouroutmemorysize;
 	char * prefix;
-	unsigned char * outbuf = NULL;
+	uint8_t * outbuf = NULL;
 	int return_code = 0;
 
 	in = TIFFOpen(infilename, "r");
@@ -771,10 +773,10 @@ makeMosaicFromTIFFFile(char * infilename)
 		overlapinpercent, &outmemorysize, &ouroutmemorysize,
 		&hnpieces, &vnpieces, &hoverlap, &voverlap);
 	if (verbose)
-		fprintf(stderr, "File \"%s\": dimensions " UINT32_FORMAT
-			" x " UINT32_FORMAT "; "
+		fprintf(stderr, "File \"%s\": dimensions %" PRIu32
+			" x %" PRIu32 "; "
 			"%u bits per sample and %u samples per pixel; "
-			"takes %.3f MiB of memory (" UINT64_FORMAT
+			"takes %.3f MiB of memory (%" PRIu64
                         " bytes) to open fully.\n",
 			infilename, inimagewidth, inimagelength,
 			bitspersample, spp,
@@ -926,12 +928,12 @@ makeMosaicFromTIFFFile(char * infilename)
 
 	if (verbose) {
 		fprintf(stderr, "File \"%s\": decided for "
-			UINT32_FORMAT " x " UINT32_FORMAT " = "
-			UINT32_FORMAT " pieces of " UINT32_FORMAT " x "
-			UINT32_FORMAT " with horiz. overlap of "
-			UINT32_FORMAT " and vert. overlap of "
-			UINT32_FORMAT " pixels that will each take at "
-			"most %.3f MiB of memory (" UINT64_FORMAT
+			"%" PRIu32 " x %" PRIu32 " = "
+			"%" PRIu32 " pieces of %" PRIu32 " x "
+			"%" PRIu32 " with horiz. overlap of "
+			"%" PRIu32 " and vert. overlap of "
+			"%" PRIu32 " pixels that will each take at "
+			"most %.3f MiB of memory (%" PRIu64
 			" bytes) to open fully.\n",
 			infilename, hnpieces, vnpieces,
 			hnpieces * vnpieces, outwidth,
@@ -1094,8 +1096,8 @@ makeMosaicFromTIFFFile(char * infilename)
 					fprintf(stderr, "Output file "
 						"\"%s\" open. Will "
 						"write piece of size "
-						UINT32_FORMAT " x "
-						UINT32_FORMAT ".\n",
+						"%" PRIu32 " x "
+						"%" PRIu32 ".\n",
 						outfilename,
 						outwidthwithoverlap,
 						outlengthwithoverlap);
@@ -1223,7 +1225,7 @@ makeMosaicFromTIFFFile(char * infilename)
 static void
 usage()
 {
-	fprintf(stderr, "tiffmakemosaic v" PACKAGE_VERSION " license GNU GPL v3 (c) 2012-2021 Christophe Deroulers\n\n");
+	fprintf(stderr, "tiffmakemosaic v" PACKAGE_VERSION " license GNU GPL v3 (c) 2012-2025 Christophe Deroulers\n\n");
 	fprintf(stderr, "Quote \"Deroulers et al., Diagnostic Pathology 2013, 8:92\" in your production\n       http://doi.org/10.1186/1746-1596-8-92\n\n");
 	fprintf(stderr, "Usage: tiffmakemosaic [options] file1.tif [file2.tif...]\n");
 	fprintf(stderr, " Produces a mosaic if needed, for each of the given files. A mosaic is a set of one-strip TIFF files of equal size, each of which has bounded size, which reproduce the original file if reassembled contiguously. Options:\n");
@@ -1673,7 +1675,7 @@ int main(int argc, char * argv[])
 				" tile dimensions but don't write files.\n");
 		if (mosaicpiecesize)
 			fprintf(stderr, "Maximum memory requirement of each "
-			"produced piece: %.3f MiB (" UINT64_FORMAT
+			"produced piece: %.3f MiB (%" PRIu64
 			" bytes).\n",
 			mosaicpiecesize / 1048576.0,
 			mosaicpiecesize);
